﻿RaidInviteHelper = LibStub("AceAddon-3.0"):NewAddon("RaidInviteHelper", "AceTimer-3.0", "AceConsole-3.0", "AceEvent-3.0")

function RaidInviteHelper:OnInitialize()
	--message("RaidInviteHelper has loaded.")
	RaidInviteHelper:RegisterChatCommand("rihtest", "TestText")
	RaidInviteHelper:RegisterChatCommand("rihinvite", "Invite")
	RaidInviteHelper:RegisterChatCommand("rihstore", "Store")
	RaidInviteHelper:RegisterChatCommand("rihadd", "Add")
	RaidInviteHelper:RegisterChatCommand("rihremove", "Remove")
	RaidInviteHelper:RegisterChatCommand("rihmembers", "Members")
	RaidInviteHelper:RegisterChatCommand("rihhelp", "Help")
	RaidInviteHelper:RegisterChatCommand("rih", "Show")
	RaidInviteHelper:RegisterChatCommand("rl", "ReloadUI")
end




--Objective of this addon
-- 1) Store a list of player names, either by pulling from the current group or manual addition.
-- 2) Show an interface displaying the players names, as well as their online/offline status.
-- 3) Allow a button to click that sends an invite to all of the players on the list that are not in the party.
-- 4) Optional: Specify a specific time and date during which to automatically invite people.


-- Store all of the current party members into tableGroupSet1
function RaidInviteHelper:Store()
	raidSize = GetNumGroupMembers();
	if raidSize == 0 then
		print("Not in a group. Moron.");
	else
		tableGroupSet1 = {}
		for i = 1, raidSize do
			local name = GetRaidRosterInfo(i)
			-- The more variables you assigne to GetRaidRosterInfo the further in its list it goes, so if you want rank, the second setting, 
			-- you have to assign two variables and the second one will be rank.
			table.insert(tableGroupSet1, name);	
		end
	end
end



-- Displays a list of all members of the tableGroupSet1
function RaidInviteHelper:Members()
	print("Displaying all members of tableGroupSet1")
	rihEditBox:SetText("")
	for i, v in ipairs(tableGroupSet1) do 
	print(v) 
	rihEditBox:SetText(rihEditBox:GetText() .. v .. "\n")
	end
	rihMemberDrop:SetList(tableGroupSet1)
end



-- Allows adding members to the tableGroupSet1 manually.
function RaidInviteHelper:Add(memberAdd)
	isDupe = false;
	if tableGroupSet1 == nil then
		tableGroupSet1 = {}
	end
	for i, v in ipairs(tableGroupSet1) do 
		if v == memberAdd then 
		isDupe = true 
		print("Already a member.")
		end
	end
	if isDupe == false then
		print("Added ", memberAdd)
		table.insert(tableGroupSet1, memberAdd);
	end
	
end



-- Allows for the removal of a member of the tableGroupSet1 manually.
-- Also rearranges the table to not leave an empty space in it.
function RaidInviteHelper:Remove(memberRemove)
	for i, v in ipairs(tableGroupSet1) do 
		if v == memberRemove then 
		table.remove(tableGroupSet1, i)
		print("Removed")
		end
	end
end



-- Invites everyone in talbeGroupSet1. Has a wait function built in to help with not being able
-- to form a group as a raid group immediately.
function RaidInviteHelper:Invite()
	print("Invite")
	self.playerName = UnitName("player")
	self.timerCount = 0
	self.groupMems = 0
	self.tableSize = table.getn(tableGroupSet1)
	self.tableIter = 1
	self.inviteTimer = self:ScheduleRepeatingTimer("InviteTime", .5)
end

-- A timer that helps deal with the lag of inviting and forming a raid group.
-- If you just iterate on the table it works so fast that WoW won't have time to conver it to a raid group.
-- This puts a .5 second delay on each invite and check to try and make up for that fact.
function RaidInviteHelper:InviteTime()
	self.currentPlayer = tableGroupSet1[self.tableIter]
	if not UnitInParty(self.currentPlayer) then
		if self.currentPlayer ~= self.playerName then
			-- print("Inviting ", self.CurrentPlayer)
			InviteUnit(self.currentPlayer)
		end
	end
	if not IsInRaid(player) then
		ConvertToRaid()
	end
	self.tableIter = self.tableIter + 1
	if self.tableIter > self.tableSize then
		print("End of invites.")
		self:CancelTimer(self.inviteTimer)
	end
end

-- Help menu detailing the commands.

function RaidInviteHelper:Help(helpArg)
	if helpArg == "" then
		print("Help for Raid Invite Helper.")
		print("The following commands currently function.")
		print("/rihstore - Stores current party/raid in a table for use during invites. Type /rihhelp store for more help.")
		print("/rihinvite - Invites all characters stored. Type /rihhelp invite for more help.")
		print("/rihadd - Add a character manually, use /rihadd CharacterName-Server . Type /rihhelp add for more help.")
		print("/rihremove - Remove a character manually, use /rihremove CharacterName-Server . Type /rihhelp remove for more help.")
		print("/rihmembers - Shows all of the current characters in the saved group. Type /rihhelp members for more help.")
	else if helpArg == "store" then
		print("/rihstore - This command will store all of the current raid or group members in a table. The table will last between session. Each use of this OVERWRITES the prevoius table. It does not matter if the player is on another server.")
	else if helpArg == "invite" then
		print("/rihinvite - This command will invite everyone in the saved table. It will also convert the group to a raid in order to accomodate all of the users. It will only send out the invite if the player is not in the group.")
	else if helpArg == "add" then
		print("/rihadd CharacterName-Server - This command manually adds a character. If they are on your server, you do not have to include the -server tag. Will not take duplicate entries.")
		print("Example: /rihadd Kechlion-Maelstrom ")
	else if helpArg == "remove" then
		print("/rihremove CharacterName-Server - This command manually removes a character. This must match what is stored in the table or it will not work. Use the /rihmembers command to list all current members.")
		print("Example: /rihremove Kechlion")
	else if helpArg == "members" then
		print("/rihmembers - This command will list out the current population of characters that have been saved. Useful if you need to remove someone and are not sure what their name is.")
	else
		Print("Invalid option.")
	end
end
end
end
end
end
end

function RaidInviteHelper:ReloadUI()
	ReloadUI()
end

-- Gui portion of the addon. Yeah... it's basic. Deal with it.
function RaidInviteHelper:Show()

-- Create the inital frame.
local AceGUI = LibStub("AceGUI-3.0")
-- https://www.wowace.com/addons/ace3/pages/ace-gui-3-0-tutorial/
rihFrame = AceGUI:Create("Frame")
rihFrame:SetTitle("Raid Invite Helper")
rihFrame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
rihFrame:SetLayout("Flow")
rihFrame:SetHeight(520)
rihFrame:SetWidth(600)

-- Create a group for the text box to go in.
rihTextGroup = AceGUI:Create("SimpleGroup")
rihTextGroup:SetLayout("Flow")
rihFrame:AddChild(rihTextGroup)

--Create a group for the buttons to go in.
rihBtnGroup = AceGUI:Create("SimpleGroup")
rihBtnGroup:SetLayout("List")
rihBtnGroup:SetWidth(220)
rihFrame:AddChild(rihBtnGroup)


-- An edit box to display all of the group members.
rihEditBox = AceGUI:Create("MultiLineEditBox")
rihEditBox:SetLabel("Raid Members")
rihEditBox:DisableButton(true)
rihEditBox:SetRelativeWidth(1)
rihEditBox:SetNumLines(30)
rihTextGroup:AddChild(rihEditBox)

-- A button to display the group members.
rihMembersBtn = AceGUI:Create("Button")
rihMembersBtn:SetText("Show Group Members")
rihMembersBtn:SetRelativeWidth(1)
rihMembersBtn:SetCallback("OnClick", function() RaidInviteHelper:Members() end)
rihBtnGroup:AddChild(rihMembersBtn)

-- A button to save the current group to the group members.
rihStoreBtn = AceGUI:Create("Button")
rihStoreBtn:SetText("Store Current Group")
rihStoreBtn:SetRelativeWidth(1)
rihStoreBtn:SetCallback("OnClick", function() RaidInviteHelper:Store() end)
rihBtnGroup:AddChild(rihStoreBtn)

-- A button to invite the current group members.
rihInviteBtn = AceGUI:Create("Button")
rihInviteBtn:SetText("Invite to Raid")
rihInviteBtn:SetRelativeWidth(1)
rihInviteBtn:SetCallback("OnClick", function() RaidInviteHelper:Invite() end)
rihBtnGroup:AddChild(rihInviteBtn)

-- A button to add a player.
rihAddBtn = AceGUI:Create("Button")
rihAddBtn:SetText("Add Target Player")
rihAddBtn:SetRelativeWidth(1)
rihAddBtn:SetCallback("OnClick", function()
	invitee = ""
	targName, targRealm = UnitName("target")
	if targName == nil then
		print("No Target")
	else if targRealm == nil then
		invitee = targName
		RaidInviteHelper:Add(invitee)
	else
		invitee = targName .. "-" .. targRealm
		RaidInviteHelper:Add(invitee)
	end
	end
end)
rihBtnGroup:AddChild(rihAddBtn)

-- A button to remove a character from the list.
rihRemoveBtn = AceGUI:Create("Button")
rihRemoveBtn:SetText("Remove from List")
rihRemoveBtn:SetRelativeWidth(1)
rihRemoveBtn:SetCallback("OnClick", function() 
	if rihMemberDrop:GetValue() ~= nil then
		print('Removing ' .. tableGroupSet1[rihMemberDrop:GetValue()])
		table.remove(tableGroupSet1, rihMemberDrop:GetValue())
		rihMemberDrop:SetValue(nil)
	else
		print("No one selected.")
	end
end)
rihBtnGroup:AddChild(rihRemoveBtn)

-- A dropdownlist showing members of the current group -if populated-.
rihMemVar = ""
rihMemberDrop = AceGUI:Create("Dropdown")
rihMemberDrop:SetLabel("Members")
rihMemberDrop:SetRelativeWidth(1)
rihMemberDrop:SetMultiselect(false)
rihBtnGroup:AddChild(rihMemberDrop)
	
end
--End of file.